﻿using System.ServiceModel;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System;


namespace Service
{
	[DataContract]
	[Serializable]
	public class PaymentDisease
	{
		[DataMember]
		public int PaymentDiseaseId { get;set;}
		[DataMember]
		public int DiseaseId { get;set;}
		[DataMember]
		public string PricePatient  { get;set;}
	}

	[DataContract]
	[Serializable]
	public class DrugForm: Drug
	{
		[DataMember]
		public int PrescriptionTypeId { get;set;}
		[DataMember]
		public int IsReimbursed { get;set;}
		[DataMember]
		public int IsNarcPsych { get;set;}
		[DataMember]
		public string DrugRegNr { get;set;}
		[DataMember]
		public string Price { get;set;}
		[DataMember]
		public string PriceLimit { get;set;}
		[DataMember]
		public int PackageId { get;set;}
		[DataMember]
		public string DrugEan { get;set;}
		[DataMember]
		public int SortForm { get;set;}
		[DataMember]
		public int SortPackage { get;set;}
		[DataMember]
		public int Bloz7 { get;set;}
		[DataMember]
		public string Modified { get;set;}
		[DataMember]
		public string Status { get;set;}
		[DataMember]
		public List<PaymentDisease> PaymentDiseases { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Drug
	{
		[DataMember]
		public int DrugId { get;set;}
		[DataMember]
		public string DrugFullName { get;set;}
		[DataMember]
		public string DrugShortName { get;set;}
		[DataMember]
		public string DrugDosage { get;set;}
		[DataMember]
		public string DrugForm { get;set;}
		[DataMember]
		public string DrugPackage { get;set;}
	}

	[DataContract]
	[Serializable]
	public class DrugListElem
	{
		[DataMember]
		public int DrugId { get;set;}
		[DataMember]
		public string DrugName { get;set;}
		[DataMember]
		public string DrugCompanyName { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Status
	{
		[DataMember]
		public int StatusId { get;set;}
		[DataMember]
		public string StatusName { get;set;}
	}

	[DataContract]
	[Serializable]
	public class PackageSubst
	{
		[DataMember]
		public int PackageId;
		[DataMember]
		public int SubstId { get;set;}
		[DataMember]
		public string SubstName { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Subst
	{
		[DataMember]
		public int SubstId { get;set;}
		[DataMember]
		public string SubstName { get;set;}
		[DataMember]
		public string Modified { get;set;}
		[DataMember]
		public int Status { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Payment
	{
		[DataMember]
		public int PaymentId { get;set;}
		[DataMember]
		public string PaymentName { get;set;}
		[DataMember]
		public string Modified { get;set;}
		[DataMember]
		public int Status { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Disease
	{
		[DataMember]
		public int DiseaseId { get;set;}
		[DataMember]
		public string DiseaseName { get;set;}
		[DataMember]
		public string DiseaseDesc { get;set;}
		[DataMember]
		public int DiseaseType { get;set;}
		[DataMember]
		public string Modified { get;set;}
		[DataMember]
		public int Status { get;set;}
	}

	[DataContract]
	[Serializable]
	public class DrugPicture
	{
		[DataMember]
		public int PackageId { get;set;}
		[DataMember]
		public string PicturePath { get;set;}
		[DataMember]
		public string PictureMd5 { get;set;}
	}

	[DataContract]
	[Serializable]
	public class Prescription
	{
		[DataMember]
		public int PrescriptionTypeId { get;set;}
		[DataMember]
		public string PrescriptionType { get;set;}
		[DataMember]
		public string PrescriptionTypeDesc { get; set; }
		[DataMember]
		public string Modified { get;set;}
		[DataMember]
		public int Status { get;set;}
	}


	[DataContract]
	[Serializable]
	public class Company
	{
		[DataMember]
		public int CompanyId { get;set;}
		[DataMember]
		public string CompanyName { get;set;}
		[DataMember]
		public string PrescriptionTypeDesc { get; set; }
	}

	[DataContract]
	[Serializable]
	public class DrugDescr
	{
		[DataMember]
		public int PackageId { get;set;}
		[DataMember]
		public string DrugText{ get; set; }
		[DataMember]
		public string DrugHtml{ get; set; }
	}

	[ServiceContract]
	public interface IWcfService
	{
		[OperationContract]
		string getVersion(string key);

		[OperationContract]
		List<Drug> getDrugFormsByDate(string key);

		[OperationContract]
		Status getStatus(string key);

		[OperationContract]
		List<Subst> getSubstAll(string key);

		[OperationContract]
		List<Payment> getDiseasePayments(string key);

		[OperationContract]
		List<Disease> getDiseases(string key);

		[OperationContract]
		List<DrugPicture> getDrugPicturesByPackageIds (string key, List<int> ids);

		[OperationContract]
		List<PackageSubst> getSubstByPackageIds(string key, List<int> ids);

		[OperationContract]
		List<Prescription> getPrescriptions (string key);

		[OperationContract]
		List<Company> getCompanyListByName (string key, string txt);

		[OperationContract]
		List<DrugForm> getDrugFormsByCompanyIds (string key, List<int> ids);

		[OperationContract]
		List<DrugDescr> getDrugDescrsByPackageIds (string key, List<int> ids);

		[OperationContract]
		List<DrugListElem> getDrugListByName (string key, string txt);
	}
}